--==============================================================================
--
--     File Name: adb3_target_inc_pkg_syn_ngc_admxrc7k1.vhd
--       Version: 0.3
--          Date: 12 Apr 2013
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Package adb3_target_inc_pkg
--
--   Description: Package used by target FPGAs during synthesis and Full MPTL
--                simulation
--
--   Limitations: ADM-XRC-7K1 synthesis/Full MPTL simulation only
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 18 Jan 2012: Initial design
--                0.2, TW, 28 May 2012:
--                  - Added 'mptl_ready' outport port to component
--                    'mptl_if_target_wrap'.
--                0.3, KDC, 12 Apr 2013:
--                  - Added new constant DDR3_BANK_ROW_WIDTH_MAX to define the
--                    size of DDR3 address bus which is driven. Constant
--                    DDR3_BANK_ROW_WIDTH defines the size that is used.
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;


package adb3_target_inc_pkg is

  -------------------------
  -- General definitions --
  -------------------------

  -- Record type for double ended signals
  type std_logic_dbl_t is record
    p : std_logic;
    n : std_logic;
  end record;


  -----------------------
  -- Clock definitions --
  -----------------------

  -- Target FPGA clock inputs
  constant REF_CLK_FREQ_HZ : natural := 200000000;

  type clks_in_t is record
    ref_clk      : std_logic_dbl_t;
    lclk         : std_logic_dbl_t;
    xrm_gclk_m2c : std_logic_dbl_t;
    ddr3_clk0    : std_logic_dbl_t;
    ddr3_clk1    : std_logic_dbl_t;
  end record;

  -- Target FPGA MGT clock inputs
  -- MGT115 CLK0            = Programmable clock 0B (derived from 25MHz oscillator)(reserved for MPTL interface use)
  -- MGT116 CLK1 (buffered) = 100MHz PCIe reference clock (derived from 100MHz input on P5)
  -- MGT117 CLK0 (buffered) = XRM reference clock
  -- MGT117 CLK1 (buffered) = Programmable clock 1 (derived from 25MHz oscillator)
  -- MGT118 CLK0 (buffered) = P6 reference clock (derived from A19/B19 input on P6)

  -- Select MGT clock inputs to buffer                        18   17   16   15   14   13   12   11   10
  constant MGT_CLKS_VALID : std_logic_vector(17 downto 0) := "01"&"11"&"10"&"00"&"00"&"00"&"00"&"00"&"00"; -- "clk1 clk0"

  type clks_mgt_in_t is record
    mgt115_clk0 : std_logic_dbl_t;
    mgt116_clk1 : std_logic_dbl_t;
    mgt117_clk0 : std_logic_dbl_t;
    mgt117_clk1 : std_logic_dbl_t;
    mgt118_clk0 : std_logic_dbl_t;
  end record;

  -- Target FPGA clock outputs
  type clks_out_t is record
    xrm_mgtclk_c2m : std_logic_dbl_t;
  end record;


  ----------------------
  -- GPIO definitions --
  ----------------------

  -- Target GPIO interfaces
  constant XRM_GPIO_WIDTH : natural := 146;
  constant PN4_GPIO_WIDTH : natural := 32; -- 0/32 Build option (p)
  constant PN6_GPIO_WIDTH : natural := 19;

  type xrm_gpio_t is record
    xrm_da_p    : std_logic_vector(15 downto 0);
    xrm_da_n    : std_logic_vector(15 downto 0);
    xrm_db_p    : std_logic_vector(15 downto 0);
    xrm_db_n    : std_logic_vector(15 downto 0);
    xrm_dc_p    : std_logic_vector(15 downto 0);
    xrm_dc_n    : std_logic_vector(15 downto 0);
    xrm_dd_p    : std_logic_vector(15 downto 0);
    xrm_dd_n    : std_logic_vector(15 downto 0);
    xrm_da_cc_p : std_logic;
    xrm_da_cc_n : std_logic;
    xrm_db_cc_p : std_logic;
    xrm_db_cc_n : std_logic;
    xrm_dc_cc_p : std_logic;
    xrm_dc_cc_n : std_logic;
    xrm_dd_cc_p : std_logic;
    xrm_dd_cc_n : std_logic;
    xrm_sa      : std_logic_vector(1 downto 0);
    xrm_sb      : std_logic_vector(1 downto 0);
    xrm_sc      : std_logic_vector(1 downto 0);
    xrm_sd      : std_logic_vector(3 downto 0);
  end record;

  type pn4_gpio_t is record
    gpio_p : std_logic_vector(PN4_GPIO_WIDTH downto 1);
    gpio_n : std_logic_vector(PN4_GPIO_WIDTH downto 1);
  end record;

  type pn6_gpio_t is record
    gpio_p : std_logic_vector(PN6_GPIO_WIDTH downto 1);
    gpio_n : std_logic_vector(PN6_GPIO_WIDTH downto 1);
  end record;

  constant XRM_GPIO_DEFAULT : xrm_gpio_t := (
    xrm_da_p    => (others => 'Z'),
    xrm_da_n    => (others => 'Z'),
    xrm_db_p    => (others => 'Z'),
    xrm_db_n    => (others => 'Z'),
    xrm_dc_p    => (others => 'Z'),
    xrm_dc_n    => (others => 'Z'),
    xrm_dd_p    => (others => 'Z'),
    xrm_dd_n    => (others => 'Z'),
    xrm_da_cc_p => 'Z',
    xrm_da_cc_n => 'Z',
    xrm_db_cc_p => 'Z',
    xrm_db_cc_n => 'Z',
    xrm_dc_cc_p => 'Z',
    xrm_dc_cc_n => 'Z',
    xrm_dd_cc_p => 'Z',
    xrm_dd_cc_n => 'Z',
    xrm_sa      => (others => 'Z'),
    xrm_sb      => (others => 'Z'),
    xrm_sc      => (others => 'Z'),
    xrm_sd      => (others => 'Z'));

  constant PN4_GPIO_DEFAULT : pn4_gpio_t := (
    gpio_p => (others => 'Z'),
    gpio_n => (others => 'Z'));

  constant PN6_GPIO_DEFAULT : pn6_gpio_t := (
    gpio_p => (others => 'Z'),
    gpio_n => (others => 'Z'));

  -- GPIO interface
  type gpio_inout_t is record
    xrm_gpio : xrm_gpio_t;
    pn4_gpio : pn4_gpio_t;
    pn6_gpio : pn6_gpio_t;
  end record;

  constant GPIO_DEFAULT : gpio_inout_t := (
    xrm_gpio => XRM_GPIO_DEFAULT,
    pn4_gpio => PN4_GPIO_DEFAULT,
    pn6_gpio => PN6_GPIO_DEFAULT);


  ------------------------
  -- Memory definitions --
  ------------------------

  -- Target FPGA DDR3 SDRAM interface
  constant DDR3_BANKS : natural := 2;

  -- Interface maximum row address width
  constant DDR3_BANK_ROW_WIDTH_MAX : natural := 14; -- Up to 2 Gib parts

  -- Interface used row address width
  constant DDR3_BANK_ROW_WIDTH : natural := 14; -- Up to 2 Gib parts = 16K row x 1K col x 8 banks x 16 bits
--  constant DDR3_BANK_ROW_WIDTH : natural := 15; -- Up to 4 Gib parts = 32K row x 1K col x 8 banks x 16 bits

  -- Interface data width
  constant DDR3_BANK_DATA_WIDTH : natural := 16; -- 1 off 16-bit parts

  -- Interface address width = Bank(3)+Row(13/14/15)+Col(10) and convert to byte addressing (+2)
  constant DDR3_BYTE_ADDR_WIDTH : natural := 3 + DDR3_BANK_ROW_WIDTH + 10 + 2;

  -- DDR3 memory interface types
  type ddr3_addr_out_t is record
    a : std_logic_vector(DDR3_BANK_ROW_WIDTH_MAX-1 downto 0);
  end record;

  type ddr3_ctrl_out_t is record
    ras_l   : std_logic;
    cas_l   : std_logic;
    we_l    : std_logic;
    dm      : std_logic_vector((DDR3_BANK_DATA_WIDTH/8)-1  downto 0);
    ba      : std_logic_vector(2 downto 0);
    reset_l : std_logic;
    odt     : std_logic_vector(0 downto 0);
    cke     : std_logic_vector(0 downto 0);
  end record;

  type ddr3_data_inout_t is record
    dq    : std_logic_vector(DDR3_BANK_DATA_WIDTH-1 downto 0);
    dqs_p : std_logic_vector((DDR3_BANK_DATA_WIDTH/8)-1 downto 0);
    dqs_n : std_logic_vector((DDR3_BANK_DATA_WIDTH/8)-1 downto 0);
  end record;

  type ddr3_clk_out_t is record
    clk_p : std_logic_vector(0 downto 0);
    clk_n : std_logic_vector(0 downto 0);
  end record;

  constant DDR3_ADDR_DEFAULT_OUT : ddr3_addr_out_t := (
    a => (others => '-'));

  constant DDR3_CTRL_DEFAULT_OUT : ddr3_ctrl_out_t := (
    ras_l   => '-',
    cas_l   => '-',
    we_l    => '-',
    dm      => (others => '-'),
    ba      => (others => '-'),
    reset_l => '0',
    odt     => (others => '-'),
    cke     => (others => '-'));

  constant DDR3_DATA_DEFAULT_INOUT : ddr3_data_inout_t := (
    dq    => (others => 'Z'),
    dqs_p => (others => 'Z'),
    dqs_n => (others => 'Z'));

  constant DDR3_CLK_DEFAULT_OUT : ddr3_clk_out_t := (
    clk_p => (others => '-'),
    clk_n => (others => '-'));

  type ddr3_clk_in_array_t     is array (0 to DDR3_BANKS-1) of std_logic_dbl_t;
  type ddr3_addr_out_array_t   is array (0 to DDR3_BANKS-1) of ddr3_addr_out_t;
  type ddr3_ctrl_out_array_t   is array (0 to DDR3_BANKS-1) of ddr3_ctrl_out_t;
  type ddr3_data_inout_array_t is array (0 to DDR3_BANKS-1) of ddr3_data_inout_t;
  type ddr3_clk_out_array_t    is array (0 to DDR3_BANKS-1) of ddr3_clk_out_t;


  -- Target FPGA memory interface
  constant MEM_BANKS  : natural := DDR3_BANKS;
  constant DDR3_BANK0 : natural range 0 to (MEM_BANKS - DDR3_BANKS) := 0;

  -- Target FPGA memory interface bank address width array
  type mem_byte_addr_width_array_t is array (0 to MEM_BANKS-1) of natural;
  constant MEM_BYTE_ADDR_WIDTH_ARRAY : mem_byte_addr_width_array_t := ((DDR3_BANKS+DDR3_BANK0-1) downto DDR3_BANK0 => DDR3_BYTE_ADDR_WIDTH);

  type mem_addr_out_t is record
    ddr3_addr_out : ddr3_addr_out_array_t;
  end record;

  type mem_ctrl_out_t is record
    ddr3_ctrl_out : ddr3_ctrl_out_array_t;
  end record;

  type mem_data_inout_t is record
    ddr3_data_inout : ddr3_data_inout_array_t;
  end record;

  type mem_clk_out_t is record
    ddr3_clk_out : ddr3_clk_out_array_t;
  end record;


  --------------------------------
  -- MPTL interface definitions --
  --------------------------------

  -- Target FPGA MPTL interface OCP channels
  constant DS_CHANNELS  : natural := 1;
  constant DMA_CHANNELS : natural := 4;
  constant DM_CHANNELS  : natural := 1;

  -- Target FPGA MPTL interface OCP channel address widths
  -- Direct Slave:      22 bits =   4 MiB (expandable using bridge paging)
  -- DMA address space: 39 bits = 512 GiB
  -- Direct Master:     64 bits =  16 EiB
  constant DS_ADDR_WIDTH  : natural range 1 to ADB3_OCP_ADDR_WIDTH := 22;
  constant DMA_ADDR_WIDTH : natural range 1 to ADB3_OCP_ADDR_WIDTH := 39;
  constant DM_ADDR_WIDTH  : natural range 1 to ADB3_OCP_ADDR_WIDTH := 64;

  -- Target FPGA MPTL interface serial width
  constant MPTL_SER_WIDTH : natural := 4;

  -- MPTL data interface
  type mptl_pins_t is record
    p : std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
    n : std_logic_vector(MPTL_SER_WIDTH-1 downto 0);
  end record;

  subtype mptl_b2t_t is mptl_pins_t;
  subtype mptl_t2b_t is mptl_pins_t;

  -- MPTL sideband interface (b2t)
  type mptl_sb_b2t_t is record
    mptl_bridge_gtp_online_l : std_logic;
  end record;

  -- MPTL sideband interface (t2b)
  type mptl_sb_t2b_t is record
    mptl_target_configured_l : std_logic;
    mptl_target_gtp_online_l : std_logic;
  end record;

  component mptl_if_target_wrap
  generic(
    enable_dm                : in    boolean := false);
  port(
    -- OCP Interface
    ocp_clk                  : in    std_logic;
    direct_slave_m2s         : out   adb3_ocp_m2sT;
    direct_slave_s2m         : in    adb3_ocp_s2mT;
    dma_channels_m2s         : out   adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
    dma_channels_s2m         : in    adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
    dma_abort                : in    std_logic_vector(DMA_CHANNELS-1 downto 0);
    direct_master_m2s        : in    adb3_ocp_m2sT := ADB3_OCP_DEFAULT_M2S;
    direct_master_s2m        : out   adb3_ocp_s2mT;
    -- MPTL Interface
    mptl_b2t                 : in    mptl_b2t_t;
    mptl_t2b                 : out   mptl_t2b_t;
    mptl_clk                 : in    std_logic_dbl_t;
    -- MPTL Initialisation
    ocp_ready                : in    std_logic;
    mptl_ready               : out   std_logic;
    mptl_sb_b2t              : in    mptl_sb_b2t_t;
    mptl_sb_t2b              : out   mptl_sb_t2b_t;
    -- MPTL Sideband
    gpio_t2b                 : in    std_logic_vector(15 downto 0);
    gpio_b2t                 : out   std_logic_vector(15 downto 0));
  end component;

end;


package body adb3_target_inc_pkg is

end;