--==============================================================================
--
--     File Name: ddr3_if_pkg_k7.vhd
--       Version: 0.3
--          Date: 17 Jan 2013
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: Package for MIG DDR3 SDRAM interface
--
--   Description: Package of constants/types used by memory interfaces
--
--   Limitations: Kintex 7 products only
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2012-2013 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 27 Mar 2012: Initial design
--                0.2, KDC, 08 Nov 2012:
--                     - Removed CL, CWL, tCKE, tPRDI, tRAS, tRCD, tRP
--                     - Added clock generic to ddr3_if_bank
--                     - Added new function conv_cl_option
--                     - Added new function conv_cwl_option
--                     - Added new function conv_t_cke_option
--                     - Added new function conv_t_ras_option
--                     - Added new function conv_t_rcd_option
--                     - Added new function conv_t_rp_option
--                     - Removed function conv_phy_bit_option
--                     - Removed function conv_clock_option
--                0.3, KDC, 17 Jan 2013:
--                     - Removed RTT_NOM
--                     - Added new function conv_rtt_nom_option
--                     - Moved BYTE_LANES constants to ddr3_if_bank
--                     - Moved DATA_CTL constants to ddr3_if_bank
--                     - Added new function conv_simulation
--                     - Changed ddr3_if_bank sim generic default to false
--                     - Added new ports ddr3_xadc_temp_in, ddr3_xadc_temp_out
--                       which are needed for MIG7 1.8 onwards to ddr3_if_bank
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

package ddr3_if_pkg is

  --------------------------------
  -- MIG DDR3 core declarations --
  --------------------------------

  -- Define clock generics for specific speed bins
  type mig_clocks_t is record
    CLKIN_PERIOD   : integer;
    REFCLK_FREQ    : real;
    CLKFBOUT_MULT  : integer;
    DIVCLK_DIVIDE  : integer;
    CLKOUT0_PHASE  : real;
    CLKOUT0_DIVIDE : integer;
    CLKOUT1_DIVIDE : integer;
    CLKOUT2_DIVIDE : integer;
    CLKOUT3_DIVIDE : integer;
    nCK_PER_CLK    : integer;
    tCK            : integer;
  end record;

  constant MIG_CLOCKS_800 : mig_clocks_t := (
    CLKIN_PERIOD   => 5000,
    REFCLK_FREQ    => 200.0,
    CLKFBOUT_MULT  => 4,
    DIVCLK_DIVIDE  => 1,
    CLKOUT0_PHASE  => 337.5,
    CLKOUT0_DIVIDE => 2,
    CLKOUT1_DIVIDE => 2,
    CLKOUT2_DIVIDE => 32,
    CLKOUT3_DIVIDE => 8,
    nCK_PER_CLK    => 4,
    tCK            => 2500);

  constant MIG_CLOCKS_1600 : mig_clocks_t := (
    CLKIN_PERIOD   => 5000,
    REFCLK_FREQ    => 200.0,
    CLKFBOUT_MULT  => 8,
    DIVCLK_DIVIDE  => 1,
    CLKOUT0_PHASE  => 337.5,
    CLKOUT0_DIVIDE => 2,
    CLKOUT1_DIVIDE => 2,
    CLKOUT2_DIVIDE => 32,
    CLKOUT3_DIVIDE => 8,
    nCK_PER_CLK    => 4,
    tCK            => 1250);


  -- Define common clock generics
  type mig_clocks_common_t is record
    IODELAY_GRP    : string(1 to 11);
    RST_ACT_LOW    : integer;
    INPUT_CLK_TYPE : string(1 to 12);
  end record;

  constant MIG_CLOCKS_COMMON : mig_clocks_common_t := (
    IODELAY_GRP    => "IODELAY_MIG",
    RST_ACT_LOW    => 0,  -- Changed from 1 (active low)
    INPUT_CLK_TYPE => "SINGLE_ENDED");


  -- Define common generics
  type mig_common_t is record
    -- Width parameters
    BANK_WIDTH          : integer;
    CK_WIDTH            : integer;
    COL_WIDTH           : integer;
    CS_WIDTH            : integer;
    nCS_PER_RANK        : integer;
    CKE_WIDTH           : integer;
    DQ_CNT_WIDTH        : integer;
    DM_WIDTH            : integer;
    DQ_WIDTH            : integer;
    DQS_WIDTH           : integer;
    DQS_CNT_WIDTH       : integer;
    DATA_WIDTH          : integer;
    DATA_BUF_ADDR_WIDTH : integer;
    PAYLOAD_WIDTH       : integer;
    MC_ERR_ADDR_WIDTH   : integer;
    BM_CNT_WIDTH        : integer;
    RANKS               : integer;
    RANK_WIDTH          : integer;
    ODT_WIDTH           : integer;
    ROW_WIDTH           : integer;
    ADDR_WIDTH          : integer;
    USE_CS_PORT         : integer;
    USE_DM_PORT         : integer;
    USE_ODT_PORT        : integer;
    MEM_ADDR_ORDER      : string(1 to 15);
    -- Mode register settings
    BURST_MODE          : string(1 to 1);
    OUTPUT_DRV          : string(1 to 4);
    RTT_WR              : string(1 to 3);
    ADDR_CMD_MODE       : string(1 to 2);
    REG_CTRL            : string(1 to 3);
    -- Part timing parameters
    tREFI               : integer;
    tZQI                : integer;
    -- IODELAY/PHY parameters
    DATA_IO_IDLE_PWRDWN : string(1 to 2);
    DATA_IO_PRIM_TYPE   : string(1 to 5);
    WRLVL               : string(1 to 2);
    ORDERING            : string(1 to 4);
    TCQ                 : integer;
    -- Debug
    DEBUG_PORT          : string(1 to 3);
  end record;

  constant MIG_COMMON : mig_common_t := (
    -- Width parameters
    BANK_WIDTH          => 3,
    CK_WIDTH            => 1,
    COL_WIDTH           => 10,
    CS_WIDTH            => 1,
    nCS_PER_RANK        => 1,
    CKE_WIDTH           => 1,
    DQ_CNT_WIDTH        => 4, -- ceil(log2(DQ_WIDTH))
    DM_WIDTH            => 2,
    DQ_WIDTH            => 16,
    DQS_WIDTH           => 2,
    DQS_CNT_WIDTH       => 1, -- ceil(log2(DQS_WIDTH))
    DATA_WIDTH          => 16,
    DATA_BUF_ADDR_WIDTH => 5, -- 4 for DATA_WIDTH=32, 5 for DATA_WIDTH=16
    PAYLOAD_WIDTH       => 16,
    MC_ERR_ADDR_WIDTH   => DDR3_BYTE_ADDR_WIDTH-1,   -- 28/29 for 2/4 Gib devices
    BM_CNT_WIDTH        => 2, -- ceil(log2(nBANK_MACHS)
    RANKS               => 1,
    RANK_WIDTH          => 1,
    ODT_WIDTH           => 1,
    ROW_WIDTH           => DDR3_BANK_ROW_WIDTH,      -- 14/15 for 2/4 Gib devices
    ADDR_WIDTH          => DDR3_BYTE_ADDR_WIDTH-1,   -- 28/29 for 2/4 Gib devices
    USE_CS_PORT         => 0,
    USE_DM_PORT         => 1,
    USE_ODT_PORT        => 1,
    MEM_ADDR_ORDER      => "ROW_BANK_COLUMN",
    -- Mode register settings
    BURST_MODE          => "8",
    OUTPUT_DRV          => "HIGH",
    RTT_WR              => "OFF",
    ADDR_CMD_MODE       => "1T",
    REG_CTRL            => "OFF",
    -- Part timing parameters
    tREFI               => 7800000,
    tZQI                => 128000000,
    -- IODELAY/PHY parameters
    DATA_IO_IDLE_PWRDWN => "ON",
    DATA_IO_PRIM_TYPE   => "HP_LP",
    WRLVL               => "ON",
    ORDERING            => "NORM",
    TCQ                 => 100,
    -- Debug
    DEBUG_PORT          => "OFF");


 -- MIG byte lanes types
  type mig_byte_lanes_t is record
    byte_lanes_b0 : std_logic_vector(3 downto 0);
    byte_lanes_b1 : std_logic_vector(3 downto 0);
    byte_lanes_b2 : std_logic_vector(3 downto 0);
    byte_lanes_b3 : std_logic_vector(3 downto 0);
    byte_lanes_b4 : std_logic_vector(3 downto 0);
  end record;
  type byte_lanes_array_t is array (natural range <>) of mig_byte_lanes_t;

 -- MIG data control types
  type mig_data_ctl_t is record
    data_ctl_b0 : std_logic_vector(3 downto 0);
    data_ctl_b1 : std_logic_vector(3 downto 0);
    data_ctl_b2 : std_logic_vector(3 downto 0);
    data_ctl_b3 : std_logic_vector(3 downto 0);
    data_ctl_b4 : std_logic_vector(3 downto 0);
  end record;
  type data_ctl_array_t is array (natural range <>) of mig_data_ctl_t;


  -- MIG phy bitlanes types
  subtype bitlanes_t is std_logic_vector(47 downto 0);
  type bitlanes_array_t is array (natural range <>) of bitlanes_t;

  type phy_bitlanes_t is record
    phy_0_bitlanes : bitlanes_t;
    phy_1_bitlanes : bitlanes_t;
    phy_2_bitlanes : bitlanes_t;
  end record;

  type mig_phy_array_t is array (natural range <>) of phy_bitlanes_t;


  -- MIG map types
  subtype addrmap_t is std_logic_vector(191 downto 0);
  type addrmap_array_t is array (natural range <>) of addrmap_t;

  type mig_map_t is record
    ck_byte_map      : std_logic_vector(143 downto 0);
    addr_map         : addrmap_t;
    bank_map         : std_logic_vector(35 downto 0);
    cas_map          : std_logic_vector(11 downto 0);
    cke_odt_byte_map : std_logic_vector(7 downto 0);
    cke_map          : std_logic_vector(95 downto 0);
    odt_map          : std_logic_vector(95 downto 0);
    cs_map           : std_logic_vector(119 downto 0);
    parity_map       : std_logic_vector(11 downto 0);
    ras_map          : std_logic_vector(11 downto 0);
    we_map           : std_logic_vector(11 downto 0);
    dqs_byte_map     : std_logic_vector(143 downto 0);
    data0_map        : std_logic_vector(95 downto 0);
    data1_map        : std_logic_vector(95 downto 0);
    data2_map        : std_logic_vector(95 downto 0);
    data3_map        : std_logic_vector(95 downto 0);
    data4_map        : std_logic_vector(95 downto 0);
    data5_map        : std_logic_vector(95 downto 0);
    data6_map        : std_logic_vector(95 downto 0);
    data7_map        : std_logic_vector(95 downto 0);
    data8_map        : std_logic_vector(95 downto 0);
    data9_map        : std_logic_vector(95 downto 0);
    data10_map       : std_logic_vector(95 downto 0);
    data11_map       : std_logic_vector(95 downto 0);
    data12_map       : std_logic_vector(95 downto 0);
    data13_map       : std_logic_vector(95 downto 0);
    data14_map       : std_logic_vector(95 downto 0);
    data15_map       : std_logic_vector(95 downto 0);
    data16_map       : std_logic_vector(95 downto 0);
    data17_map       : std_logic_vector(95 downto 0);
    mask0_map        : std_logic_vector(107 downto 0);
    mask1_map        : std_logic_vector(107 downto 0);
  end record;

  type mig_map_array_t is array (natural range <>) of mig_map_t;


  component ddr3_if_bank
  generic(
    sim   : boolean := false;
    clock : mig_clocks_t := MIG_CLOCKS_1600;
    bank  : integer range 0 to DDR3_BANKS-1 := 0);
  port(
    -- OCP interface
    ocp_rst               : in    std_logic;
    ocp_clk               : in    std_logic;
    ocp_m2s               : in    adb3_ocp_m2sT;
    ocp_s2m               : out   adb3_ocp_s2mT;
    -- Memory interface
    ddr3_rst              : in    std_logic;
    ddr3_ref_clk          : in    std_logic;
    ddr3_clk              : in    std_logic;
    ddr3_iodelay_ctrl_rdy : in    std_logic;
    ddr3_xadc_temp_in     : in    std_logic_vector(11 downto 0) := (others => '0');
    ddr3_xadc_temp_out    : out   std_logic_vector(11 downto 0);
    -- Memory status
    ddr3_if_rdy           : out   std_logic;
    ddr3_if_stat          : out   std_logic_vector(3 downto 0);
    ddr3_if_err           : out   std_logic_vector(3 downto 0);
    -- Physical memory interface
    ddr3_addr_out         : out   ddr3_addr_out_t;
    ddr3_ctrl_out         : out   ddr3_ctrl_out_t;
    ddr3_data_inout       : inout ddr3_data_inout_t;
    ddr3_clk_out          : out   ddr3_clk_out_t;
    -- Debug info
    ddr3_if_debug         : out   std_logic_vector(31 downto 0));
  end component;


  -- Functions used to resolve settings vs size of DDR3 SDRAM parts
  function conv_t_rfc_option(
    constant val: natural)
  return integer;

  -- Functions used to resolve settings vs speed of operation of DDR3 SDRAM parts
  function conv_cl_option(
    constant val: natural)
  return integer;

  function conv_cwl_option(
    constant val: natural)
  return integer;

  function conv_rtt_nom_option(
    constant val: natural)
  return string;

  function conv_t_cke_option(
    constant val: natural)
  return integer;

  function conv_t_faw_option(
    constant val: natural)
  return integer;

  function conv_t_ras_option(
    constant val: natural)
  return integer;

  function conv_t_rcd_option(
    constant val: natural)
  return integer;

  function conv_t_rp_option(
    constant val: natural)
  return integer;

  function conv_t_rrd_option(
    constant val: natural)
  return integer;

  -- Functions used to resolve settings for initialisation/calibration/simulation
  function conv_sim_bypass_init_cal(
    constant sim: boolean)
  return string;

  function conv_simulation(
    constant sim: boolean)
  return string;

  function conv_temp_mon_en(
    constant sim: boolean)
  return string;

end;


package body ddr3_if_pkg is

  function conv_t_rfc_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 14)or(val = 15)
      report "(conv_t_rfc_option): DDR3 row width " & natural'image(val) & " is out of range 14..15."
      severity failure;

    case val is
      when 14     => return 160000; -- 160 ns for 2 Gib devices
      when 15     => return 260000; -- 260 ns for 4 Gib devices
      when others => return -1;
    end case;
  end;

  function conv_cl_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_cl_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500ps."
      severity failure;

    case val is
      when 1250   => return 11; -- DDR3-1600
      when 2500   => return 6;  -- DDR3-800
      when others => return -1;
    end case;
  end;

  function conv_cwl_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_cwl_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500ps."
      severity failure;

    case val is
      when 1250   => return 8; -- DDR3-1600
      when 2500   => return 5; -- DDR3-800
      when others => return -1;
    end case;
  end;

  function conv_rtt_nom_option(
    constant val: natural)
  return string is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_rtt_nom_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500 ps."
      severity failure;

    case val is
      when 1250   => return "40"; -- DDR3-1600
      when 2500   => return "60"; -- DDR3-800
      when others => return "";
    end case;
  end;

  function conv_t_cke_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_t_cke_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500 ps."
      severity failure;

    case val is
      when 1250   => return 5000; -- DDR3-1600
      when 2500   => return 7500; -- DDR3-800
      when others => return -1;
    end case;
  end;

  function conv_t_faw_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_t_faw_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500ps."
      severity failure;

    case val is
      when 1250   => return 40000; -- DDR3-1600 (2KB page size)
      when 2500   => return 50000; -- DDR3-800  (2KB page size)
      when others => return -1;
    end case;
  end;

  function conv_t_ras_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_t_ras_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500 ps."
      severity failure;

    case val is
      when 1250   => return 35000; -- DDR3-1600
      when 2500   => return 37500; -- DDR3-800
      when others => return -1;
    end case;
  end;

  function conv_t_rcd_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_t_rcd_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500 ps."
      severity failure;

    case val is
      when 1250   => return 13750; -- DDR3-1600 CL 11
      when 2500   => return 15000; -- DDR3-800  CL 6
      when others => return -1;
    end case;
  end;

  function conv_t_rp_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_t_rp_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500 ps."
      severity failure;

    case val is
      when 1250   => return 13750; -- DDR3-1600 CL 11
      when 2500   => return 15000; -- DDR3-800  CL 6
      when others => return -1;
    end case;
  end;

  function conv_t_rrd_option(
    constant val: natural)
  return integer is
  begin
    assert (val = 1250)or(val = 2500)
      report "(conv_t_rrd_option): DDR3 ck period " & natural'image(val) & " is not 1250/2500ps."
      severity failure;

    case val is
      when 1250   => return 7500;  -- DDR3-1600 (2KB page size)
      when 2500   => return 10000; -- DDR3-800  (2KB page size)
      when others => return -1;
    end case;
  end;

  function conv_sim_bypass_init_cal(
    constant sim: boolean)
  return string is
  begin
    if sim then
      return "FAST";
    else
      return "OFF";
    end if;
  end;

  function conv_simulation(
    constant sim: boolean)
  return string is
  begin
    if sim then
      return "TRUE";
    else
      return "FALSE";
    end if;
  end;

  function conv_temp_mon_en(
    constant sim: boolean)
  return string is
  begin
    if sim then
      return "OFF";
    else
      return "ON";
    end if;
  end;

end;