#
# Script for generating MIG7 v1.8 DDR3 interface using ISE 14.4 onwards
#
# Usage: xtclsh gen_mem_if_mig7_v1_8.tcl <model>
# eg.
#        xtclsh gen_mem_if_mig7_v1_8.tcl admxrc7k1
#
set mig_ver 1_8
set ise_1st 14.4
#
proc show_usage { } {
  global mig_ver
  global ise_1st
  puts "Usage:"
  puts ""
  puts "  xtclsh gen_mem_if_mig7_v${mig_ver}.tcl -? | -h | --help"
  puts "  xtclsh gen_mem_if_mig7_v${mig_ver}.tcl <model>"
  puts ""
  puts "-? -h --help      Show this message."
  puts ""
  puts "Note:"
  puts ""
  puts "  Requires ISE ${ise_1st} or later."
}
#
for { set i 0 } { $i < $argc } { incr i } {
  set argvi [ lindex $argv $i ];
  if { [ string compare ${argvi} {-?} ] == 0 || [ string compare ${argvi} {-h} ] == 0 || [ string compare ${argvi} {--help} ] == 0 } {
    show_usage
    exit 0
  }
  if { [ string index ${argvi} 0 ] == {-} } {
    puts "*** Unrecognised option '${argvi}'"
    show_usage
    exit -1
  } else {
    break
  }
}
if { $i < $argc } {
  set _Model [ lindex $argv $i ]
} else {
  puts {*** Missing model. Please specify a model, e.g. admxrc7k1.}
  show_usage
  exit -1
}
#
#
# Get ISE version from fileset.txt
if {[info exists env(XILINX)]} {
  set _Native [ file nativename $env(XILINX) ]
  set _ISE_Path ${_Native}/fileset.txt
} else {
  puts "ERROR: %XILINX% environmental variable is not set."
  exit -1
}
#
#
# Get read fileset.txt
if {[file readable ${_ISE_Path}]} {
  set ise_file [open "${_ISE_Path}" r]
  set file_data [read $ise_file]
  close $ise_file
} else {
  puts "ERROR: Cannot read file '${_ISE_Path}'."
  exit -1
}
#
#
# search fileset.txt
puts ""
puts "Searching ${_ISE_Path} for ISE version..."
puts ""
set found 0
set data [split $file_data "\n"]
foreach line $data {
  if {[regexp -nocase " version=(.*)" ${line} result ise_num]} {
    incr found
#    puts "  ($found): ${result}, ${ise_num}"
  }
}

if {${found} eq 1} {
  puts "Found ISE version ${ise_num}"
} else {
  puts "Error: Found more than 1 (${found}) version entries in ${_ISE_Path}"
  exit -1
}

if {${ise_num} < ${ise_1st}} {
  puts "Error: MIG7 v${mig_ver} requires ISE ${ise_1st} or later."
  exit -1
}
#
#
# Ensure that the current working directory is that of this script
set _script_path [ file dirname [ info script ] ]
cd ${_script_path}
#
set _CGP_Path ./${_Model}/${_Model}.cgp
set _XCO_Path ./${_Model}/mig7_v${mig_ver}/${_Model}.xco
set _PRJ_Path ./${_Model}/mig7_v${mig_ver}/${_Model}.prj
#set _CUS_Path ./custom_part
#
#
#    [file exists ${_CUS_Path}]&&
if {[file exists ${_XCO_Path}]&&
    [file exists ${_CGP_Path}]&&
    [file exists ${_PRJ_Path}]} {
  puts ""
  puts "MIG7 v${mig_ver} selected."
  puts "Model ${_Model} selected."
} else {
  puts ""
  puts "MIG7 v${mig_ver} selected."
  puts "Missing files for model '${_Model}':"
  puts ""
  puts "CGP file: ${_CGP_Path}"
  puts "XCO file: ${_XCO_Path}"
  puts "PRJ file: ${_PRJ_Path}"
#  puts "Part dir: ${_CUS_Path}"
  puts ""
  puts "Please specify a valid model, e.g. admxrc7k1."
  puts ""
  exit -1
}
#
#
# Get version number from project file
set prj_file [open "${_PRJ_Path}" r]
set file_data [read $prj_file]
close $prj_file

puts ""
puts "Searching ${_PRJ_Path} for MIG7 version..."
puts ""
set found 0
set data [split $file_data "\n"]
foreach line $data {
  if {[regexp -nocase "<version>(.*)</version>" ${line} result ver_num]} {
    incr found
#    puts "  ($found): ${result}, ${ver_num}"
    set _MIG_sub [regsub -all {\.} ${ver_num} "_"]
  }
}

if {${_MIG_sub} ne ${mig_ver}} {
  puts "Error: MIG7 version in ${_PRJ_Path} (${_MIG_sub}) is different from ${mig_ver}"
  exit -1
}

if {${found} eq 1} {
  set _MIG_Version mig_v[regsub -all {\.} ${ver_num} "_"]
  puts "Selecting MIG7 version ${ver_num} (${_MIG_Version})"
} else {
  puts "Error: Found more than 1 (${found}) version entries in ${_PRJ_Path}"
  exit -1
}
#
#
set _MIG_Temp ./${_Model}/mig7_v${mig_ver}/mig_temp
#
set _CGP_Dest ${_Model}.cgp
set _PRJ_Dest ${_MIG_Version}/user_design/mig.prj
set _XCO_Dest ${_Model}.xco
#set _CUS_Dest custom_part
#
#
puts ""
puts "Initialising project files..."
puts ""
puts "Deleting temp dir ${_MIG_Temp}"
file delete -force ${_MIG_Temp}
puts "Making temp dir ${_MIG_Temp}/${_MIG_Version}/user_design"
file mkdir ${_MIG_Temp}/${_MIG_Version}/user_design
puts "Copying CGP file ${_CGP_Path} to ${_MIG_Temp}/${_CGP_Dest}"
file copy ${_CGP_Path} ${_MIG_Temp}/${_CGP_Dest}
puts "Copying PRJ file ${_PRJ_Path} to ${_MIG_Temp}/${_PRJ_Dest}"
file copy ${_PRJ_Path} ${_MIG_Temp}/${_PRJ_Dest}
puts "Copying XCO file ${_XCO_Path} to ${_MIG_Temp}/${_XCO_Dest}"
file copy ${_XCO_Path} ${_MIG_Temp}/${_XCO_Dest}
#puts "Copying custom parts ${_CUS_Path} to ${_MIG_Temp}/${_CUS_Dest}"
#file copy ${_CUS_Path} ${_MIG_Temp}/${_CUS_Dest}
#
#
puts ""
puts "Generating MIG7 files:"
puts ""
puts "Executing coregen -r -b ${_XCO_Dest}  -p ${_CGP_Dest}"
#
cd ${_MIG_Temp}
if { [catch { exec coregen -r -b ${_XCO_Dest} -p ${_CGP_Dest} >@ stdout } msg] } {
  if { $errorCode ne "NONE" } {
    puts ""
    puts "CORE Generator error detected:"
    puts ""
    puts "  errorCode: '$errorCode'"
    puts "  errorInfo: '$errorInfo'"
    cd ../..
    exit -1
  }
}
pwd
cd ../../..
pwd
#
#
set _RTL_Path ./${_Model}/mig7_v${mig_ver}/rtl
puts ""
puts "Copying generated VHDL to RTL dir ${_RTL_Path}"
puts ""
puts "Deleting old rtl dir ${_RTL_Path}"
file delete -force ${_RTL_Path}
puts "Making rtl dir ${_RTL_Path}"
file mkdir ${_RTL_Path}
puts "Copying rtl files ${_MIG_Temp}/${_MIG_Version}/user_design/rtl to ${_RTL_Path}"
foreach _RTL_Dir [glob -directory ${_MIG_Temp}/${_MIG_Version}/user_design/rtl *] {
  file copy ${_RTL_Dir} ${_RTL_Path}
}
puts "Copying log file ${_MIG_Temp}/coregen.log to ${_RTL_Path}"
file copy ${_MIG_Temp}/coregen.log ${_RTL_Path}
#
puts ""
puts "Generation of MIG7 core ${_MIG_Version} for ${_Model} is complete."
puts ""
puts "Please install patch 53420 for model '${_Model}' by following"
puts "instructions in '${_Model}/mig7_v1_8/calibration_patch_ar53420.txt'."
puts ""

exit 0