--==============================================================================
--
--     File Name: mig_ddr3_lwb.vhd
--       Version: 0.1
--          Date: 13 Mar 2012
--         Model: Behavioral (NOT SYNTHESISABLE)
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design:
--
--   Description: Xilinx DDR3 MIG and DDR3 SDRAM light weight behavioural block
--
--   Limitations: Light weight behavioural simulation
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2012 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 13 Mar 2012: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;

library std;
use std.textio.all;

library work;
use work.ddr3_if_pkg.all;
use work.ddr3_sdram_int_pkg.all;


entity mig_ddr3_lwb is
  generic(
    message_level : natural range 0 to 7 := 0;
    app_addr_width : natural;
    app_data_width : natural;
    app_mask_width : natural);
  port(
    rst               : in    std_logic;
    clk               : in    std_logic;
    -- MIG user interface inputs
    app_addr          : in    std_logic_vector(app_addr_width-1 downto 0);
    app_cmd           : in    std_logic_vector(2 downto 0);
    app_en            : in    std_logic;
    app_sz            : in    std_logic;
    app_wdf_data      : in    std_logic_vector(app_data_width-1 downto 0);
    app_wdf_end       : in    std_logic;
    app_wdf_mask      : in    std_logic_vector(app_mask_width-1 downto 0);
    app_wdf_wren      : in    std_logic;
    -- MIG user interface outputs
    dfi_init_complete : out   std_logic;
    app_rd_data       : out   std_logic_vector(app_data_width-1 downto 0);
    app_rd_data_valid : out   std_logic;
    app_rdy           : out   std_logic;
    app_wdf_rdy       : out   std_logic;
    -- File control
    init_start        : in    boolean := false;
    init_filename     : in    string := "init.txt";
    log_start         : in    boolean := false;
    log_filename      : in    string := "log.txt");
  end;


architecture behavioural of mig_ddr3_lwb is

  constant BURST_TYPE : string(1 to 3) := "SEQ";  -- "SEQ" "INT"

  -- Message reporting (warnings, errors, failures are always reported)
  --
  -- 7 Write and read and info additional messages
  -- 6 Read and info additional messages
  -- 5 Write and info additional messages
  -- 4 Write and read additional messages
  -- 3 Info additional messages only
  -- 2 Read additional messages only
  -- 1 Write additional messages only
  -- 0 No additional messages

  constant MESSAGE_WR : boolean := (message_level=1)or(message_level=4)or(message_level=5)or(message_level=7);
  constant MESSAGE_RD : boolean := (message_level=2)or(message_level=4)or(message_level=6)or(message_level=7);

  -- Define constants for part size parameters
  constant DATA_WIDTH : natural := MIG_COMMON.DQ_WIDTH;
  constant DATA_BYTES : natural := MIG_COMMON.DQS_WIDTH;
  constant BYTE_WIDTH : natural := DATA_WIDTH/DATA_BYTES;

  constant AROW_WIDTH : natural := MIG_COMMON.ROW_WIDTH;
  constant ACOL_WIDTH : natural := MIG_COMMON.COL_WIDTH;
  constant ABAN_WIDTH : natural := MIG_COMMON.BANK_WIDTH;

  constant NUM_ROWS  : natural := 2**AROW_WIDTH;
  constant NUM_COLS  : natural := 2**ACOL_WIDTH;
  constant NUM_BANKS : natural := 2**ABAN_WIDTH;

  subtype data_s is std_logic_vector(DATA_WIDTH-1 downto 0);
  subtype arow_s is std_logic_vector(AROW_WIDTH-1 downto 0);
  subtype acol_s is std_logic_vector(ACOL_WIDTH-1 downto 0);
  subtype bank_s is std_logic_vector(ABAN_WIDTH-1 downto 0);
  subtype byte_s is std_logic_vector(DATA_BYTES-1 downto 0);

  -- Define SRAM col type = col array of data_s
  type col_array_t is array (natural range 0 to (NUM_COLS-1)) of data_s;

  -- Define pointer type to memory row
  type row_pointer_t is access col_array_t;

  -- Define SRAM bank type = row array of col_array_t
  type bank_array_t is array (natural range 0 to (NUM_ROWS-1)) of row_pointer_t;

  -- Define SRAM memory type = bank array of bank_array_t
  type data_array_t is array (natural range 0 to (NUM_BANKS-1)) of bank_array_t;

  shared variable memory : data_array_t;

  -- Define counters for number of active memory rows
  type bank_cnt_row_t is array (natural range 0 to (NUM_BANKS-1)) of natural;
  signal memory_row_cnt : bank_cnt_row_t := (others => 0);
  signal init_row_cnt   : bank_cnt_row_t := (others => 0);

  -- Define constants for commands derived from
  -- app_en & app_cmd & app_sz & app_wdf_wren & app_wdf_end;
  constant CMD_W_FIRST  : std_logic_vector(4 downto 0) := "10110";
  constant CMD_W_SECOND : std_logic_vector(4 downto 0) := "0--11";
  constant CMD_W_ONE    : std_logic_vector(4 downto 0) := "10011";
  constant CMD_R_TWO    : std_logic_vector(4 downto 0) := "11100";
  constant CMD_R_ONE    : std_logic_vector(4 downto 0) := "11000";
  constant CMD_IDLE     : std_logic_vector(4 downto 0) := "0---0";

  -- Define command types
  type command_t is (
    IDLE,
    W_FIRST,
    W_SECOND,
    W_ONE,
    R_TWO,
    R_ONE,
    ILGL);

  signal app_cmd_dec : command_t := IDLE;
  signal mig_cmd_dec : command_t := IDLE;

  signal app_cmd_vec : std_logic_vector(4 downto 0);

  -- MIG Command FIFO
  constant MIG_ADDR_WIDTH : natural := AROW_WIDTH + ACOL_WIDTH + ABAN_WIDTH;
  constant MIG_CMD_DEPTH  : natural := 3;
  constant MIG_CMD_WIDTH  : natural := 5 + MIG_ADDR_WIDTH + app_mask_width + app_data_width;

  signal mig_cmd_wr       : std_logic;
  signal mig_cmd_adv      : std_logic;
  signal mig_cmd_not_full : std_logic;
  signal mig_cmd_valid    : std_logic;
  signal mig_cmd_d        : std_logic_vector(MIG_CMD_WIDTH-1 downto 0);
  signal mig_cmd_q        : std_logic_vector(MIG_CMD_WIDTH-1 downto 0);

  signal mig_cmd_vec  : std_logic_vector(4 downto 0);
  signal mig_cmd_addr : std_logic_vector(MIG_ADDR_WIDTH-1 downto 0);
  signal mig_cmd_mask : std_logic_vector(app_mask_width-1 downto 0);
  signal mig_cmd_data : std_logic_vector(app_data_width-1 downto 0);

  signal burst_mask_d1 : std_logic_vector(app_mask_width-1 downto 0);
  signal burst_data_d1 : std_logic_vector(app_data_width-1 downto 0);

  -- User interface FSM
  type ui_state_t is (ui_s_idle,
                      ui_s_w_first,
                      ui_s_w_second,
                      ui_s_w_one,
                      ui_s_r_first,
                      ui_s_r_second,
                      ui_s_r_one);

  signal ui_state : ui_state_t;
  signal ui_wr_en : std_logic;
  signal ui_rd_en : std_logic;
  signal ui_addr  : std_logic_vector(MIG_ADDR_WIDTH-1 downto 0);
  signal ui_adv   : std_logic;

  signal n_ui_state : ui_state_t;
  signal n_ui_wr_en : std_logic;
  signal n_ui_rd_en : std_logic;
  signal n_ui_addr  : std_logic_vector(MIG_ADDR_WIDTH-1 downto 0);
  signal n_ui_adv   : std_logic;

  -- Input retimes
  signal write_en     : boolean;
  signal read_en      : boolean;
  signal start_bank   : bank_s;
  signal start_row    : arow_s;
  signal start_col    : acol_s;
  signal start_burst  : std_logic_vector(2 downto 0);
  signal start_bl_cyc : integer range BL_MIN to BL_MAX;
  signal start_cyc    : integer;

  -- Output retimes
  signal data_out_full : std_logic_vector(app_data_width-1 downto 0);

  -- Test data write/read
  -- 256/32 = 8 locations. 1 cycle = BL8.
  -- 128/32 = 4 locations. 1 cycle = BC4, 2 cycles = BL8.
  -- 128/16 = 8 locations. 1 cycle = BL8.
  constant DATA_WORDS : natural := app_data_width/DATA_WIDTH;

  type data_n_s is array  (natural range 0 to DATA_WORDS-1) of natural;
  type mem_mask_array_t is array (natural range 0 to DATA_WORDS-1) of byte_s;
  type mem_data_array_t is array (natural range 0 to DATA_WORDS-1) of data_s;

  signal memory_wr      : std_logic;
  signal memory_wr_bank : data_n_s;
  signal memory_wr_row  : data_n_s;
  signal memory_wr_col  : data_n_s;
  signal memory_wr_mask : mem_mask_array_t;
  signal memory_wr_data : mem_data_array_t;

  signal memory_rd      : std_logic;
  signal memory_rd_bank : data_n_s;
  signal memory_rd_row  : data_n_s;
  signal memory_rd_col  : data_n_s;
  signal memory_rd_data : mem_data_array_t;

  component mig_ddr3_lwb_cmd_fifo
  generic(
    ORDER     : in    natural range 1 to 6;
    WIDTH     : in    natural;
    EMPTY_VAL : in    std_logic := '0';
    EMPTY_LO  : in    natural := 0;
    EMPTY_HI  : in    natural := 0);
  port(
    rst       : in    std_logic;
    clk       : in    std_logic;
    wr        : in    std_logic;
    d         : in    std_logic_vector(WIDTH - 1 downto 0);
    adv       : in    std_logic;
    full      : out   std_logic;
    not_full  : out   std_logic;
    valid     : out   std_logic;
    q         : out   std_logic_vector(WIDTH - 1 downto 0));
  end component;

begin

  app_cmd_vec <= app_en & app_cmd(0) & app_sz & app_wdf_wren & app_wdf_end;

  -- Decode app_cmd_vec to command type
  app_cmd_dec <= W_FIRST  when (std_match(app_cmd_vec,CMD_W_FIRST)) else
                 W_SECOND when (std_match(app_cmd_vec,CMD_W_SECOND)) else
                 W_ONE    when (std_match(app_cmd_vec,CMD_W_ONE)) else
                 R_TWO    when (std_match(app_cmd_vec,CMD_R_TWO)) else
                 R_ONE    when (std_match(app_cmd_vec,CMD_R_ONE)) else
                 IDLE     when (std_match(app_cmd_vec,CMD_IDLE)) else
                 ILGL;

  -- Write valid MIG command/data to FIFO
  mig_cmd_wr  <= '1' when ((mig_cmd_not_full='1')and(app_cmd_dec /= IDLE)and(app_cmd_dec /= ILGL)) else '0';

  mig_cmd_d   <= app_en & app_cmd(0) & app_sz & app_wdf_wren & app_wdf_end &
                 app_addr(MIG_ADDR_WIDTH-1 downto 0) & app_wdf_mask & app_wdf_data;
  mig_cmd_adv <= n_ui_adv ;

  mig_ddr3_lwb_cmd_fifo_i : mig_ddr3_lwb_cmd_fifo
  generic map(
    ORDER     => MIG_CMD_DEPTH,
    WIDTH     => MIG_CMD_WIDTH,
    EMPTY_VAL => '0',
    EMPTY_LO  => 0,
    EMPTY_HI  => MIG_CMD_WIDTH)
  port map(
    rst      => rst,
    clk      => clk,
    wr       => mig_cmd_wr,
    d        => mig_cmd_d,
    adv      => mig_cmd_adv,
    full     => open,
    not_full => mig_cmd_not_full,
    valid    => mig_cmd_valid,
    q        => mig_cmd_q);

  app_rdy     <= mig_cmd_not_full when (rst = '0') else '0';
  app_wdf_rdy <= mig_cmd_not_full when (rst = '0') else '0';

  -- Error checking
  mig_cmd_err_p : process(clk)
  begin
    if rising_edge(clk) then
      if (mig_cmd_wr = '1')and(mig_cmd_not_full = '0') then
        assert false
          report "(mig_ddr3_lwb): Attempt to write to mig_cmd_fifo when full"
          severity error;
      end if;
    end if;
  end process;

  mig_cmd_vec  <= mig_cmd_q(MIG_CMD_WIDTH-1 downto MIG_CMD_WIDTH-5) when (ui_adv = '1') else CMD_IDLE;
  mig_cmd_addr <= mig_cmd_q(MIG_CMD_WIDTH-6 downto app_mask_width+app_data_width);
  mig_cmd_mask <= mig_cmd_q(app_mask_width+app_data_width-1 downto app_data_width);
  mig_cmd_data <= mig_cmd_q(app_data_width-1 downto 0);

  mig_cmd_dec <= W_FIRST  when (std_match(mig_cmd_vec,CMD_W_FIRST)) else
                 W_SECOND when (std_match(mig_cmd_vec,CMD_W_SECOND)) else
                 W_ONE    when (std_match(mig_cmd_vec,CMD_W_ONE)) else
                 R_TWO    when (std_match(mig_cmd_vec,CMD_R_TWO)) else
                 R_ONE    when (std_match(mig_cmd_vec,CMD_R_ONE)) else
                 IDLE     when (std_match(mig_cmd_vec,CMD_IDLE)) else
                 ILGL;


  ------------------- Input Retime Section ---------------------

  in_retime : process(rst,clk)
  begin
    if rst = '1' then
      burst_data_d1 <= (others => '0');
      burst_mask_d1 <= (others => '0');
    elsif rising_edge(clk) then
      burst_data_d1 <= mig_cmd_data;
      burst_mask_d1 <= mig_cmd_mask;
    end if;
  end process;


  ------------------- Output Retime Section ---------------------

  out_retime : process(rst,clk)
  begin
    if rst = '1' then
      dfi_init_complete <= '0';
      app_rd_data       <= (others => '-');
      app_rd_data_valid <= '0';
    elsif rising_edge(clk) then
      dfi_init_complete <= '1';
      app_rd_data       <= data_out_full;
      app_rd_data_valid <= memory_rd;
    end if;
  end process;


  ------------------- Memory Write/Read Preparation Section ---------------------



  ---------------------------------------------------------------------------
  -- Command Check FSM
  ---------------------------------------------------------------------------

  gen_ui_check : process(rst,clk)
  begin
    if rst = '1' then
      null;
    elsif rising_edge(clk) then
      -- Report illegal app commands after reset off
      assert not(app_cmd_dec = ILGL)
        report "(mig_ddr3_lwb): ERROR: Illegal UI command (app_en=" & std_logic'image(app_en) &
                                                        ",app_cmd=" & std_logic'image(app_cmd(0)) &
                                                         ",app_sz=" & std_logic'image(app_sz) &
                                                   ",app_wdf_wren=" & std_logic'image(app_wdf_wren) &
                                                    ",app_wdf_end=" & std_logic'image(app_wdf_end) & ")"
        severity error;

      -- Report illegal FIFO commands after reset off
      assert not(mig_cmd_dec = ILGL)
        report "(mig_ddr3_lwb): ERROR: Illegal FIFO command (app_en=" & std_logic'image(mig_cmd_vec(4)) &
                                                          ",app_cmd=" & std_logic'image(mig_cmd_vec(3)) &
                                                           ",app_sz=" & std_logic'image(mig_cmd_vec(2)) &
                                                     ",app_wdf_wren=" & std_logic'image(mig_cmd_vec(1)) &
                                                      ",app_wdf_end=" & std_logic'image(mig_cmd_vec(0)) & ")"
        severity error;

      -- Report illegal 2 cycle access if DATA_WORDS = 8
      assert not((DATA_WORDS = 8)and((mig_cmd_dec = W_FIRST)or(mig_cmd_dec = W_SECOND)or(mig_cmd_dec = R_TWO)))
        report "(mig_ddr3_lwb): ERROR: 2 cycle command illegal with DATA_WORDS = 8."
        severity error;

      -- Report illegal (7 series only) 1 cycle access if DATA_WORDS = 4
      assert not((DATA_WORDS = 4)and((mig_cmd_dec = W_ONE)or(mig_cmd_dec = R_ONE)))
        report "(mig_ddr3_lwb): ERROR: 1 cycle command illegal (7 series) with DATA_WORDS = 4."
        severity error;

      -- Report out of sequence MIG commands
      case ui_state is
        when ui_s_w_first =>  -- Write 1st cycle
          if (mig_cmd_dec = W_SECOND) then
            null;
          else
            assert false
              report "(mig_ddr3_lwb): ERROR: Expecting write cycle 2 - Command out of sequence."
              severity error;
          end if;

        when ui_s_r_first =>  -- Read 1st cycle
          if (mig_cmd_dec = IDLE) then
            null;
          else
            assert false
              report "(mig_ddr3_lwb): ERROR: Expecting Read cycle 2 (Idle) - Command out of sequence."
              severity error;
          end if;

        -- ui_s_idle     : Idle
        -- ui_s_w_second : Write 2nd cycle
        -- ui_s_w_one    : Write cycle
        -- ui_s_r_second : Read 2nd cycle
        -- ui_s_r_one    : Read cycle
        when others =>
          if (mig_cmd_dec = W_SECOND) then
            assert false
              report "(mig_ddr3_lwb): ERROR: Write 2nd cycle out of sequence."
              severity error;
          else
            null;
          end if;
      end case;
    end if;
  end process;


  gen_ui_state : process(rst,clk)
  begin
    if rst = '1' then
      ui_state <= ui_s_idle;
      ui_wr_en <= '0';
      ui_rd_en <= '0';
      ui_addr  <= (others => '-');
      ui_adv   <= '0';
    elsif rising_edge(clk) then
      ui_state <= n_ui_state;
      ui_wr_en <= n_ui_wr_en;
      ui_rd_en <= n_ui_rd_en;
      ui_addr  <= n_ui_addr;
      ui_adv   <= n_ui_adv;
    end if;
  end process;


  gen_n_ui_state : process(
    ui_state,
    ui_addr,
    mig_cmd_dec,
    mig_cmd_addr)
  begin
    case ui_state is
      when ui_s_w_first =>  -- Write 1st cycle
        n_ui_addr <= ui_addr;

        if (mig_cmd_dec = W_SECOND) then
          n_ui_state <= ui_s_w_second;
          n_ui_wr_en <= '1';
          n_ui_rd_en <= '0';
          n_ui_adv   <= '1';
        else -- Error
          n_ui_state <= ui_s_idle;
          n_ui_wr_en <= '0';
          n_ui_rd_en <= '0';
          n_ui_adv   <= '0';
        end if;

      when ui_s_r_first =>  -- Read 1st cycle
        n_ui_addr <= ui_addr;

        if (mig_cmd_dec = IDLE) then
          n_ui_state <= ui_s_r_second;
          n_ui_wr_en <= '0';
          n_ui_rd_en <= '1';
          n_ui_adv   <= '1';
        else -- Error
          n_ui_state <= ui_s_idle;
          n_ui_wr_en <= '0';
          n_ui_rd_en <= '0';
          n_ui_adv   <= '0';
        end if;

      -- ui_s_idle,
      -- ui_s_w_second,
      -- ui_s_w_one,
      -- ui_s_r_second,
      -- ui_s_r_one
      when others =>
        n_ui_addr <= mig_cmd_addr;  -- Initialise 4-byte address

        case mig_cmd_dec is
          when W_FIRST =>
            n_ui_state <= ui_s_w_first;
            n_ui_wr_en <= '1';
            n_ui_rd_en <= '0';
            n_ui_adv   <= '1';
          when W_SECOND =>
            n_ui_state <= ui_s_idle; -- Error
            n_ui_wr_en <= '1';
            n_ui_rd_en <= '0';
            n_ui_adv   <= '1';
          when W_ONE =>
            n_ui_state <= ui_s_w_one;
            n_ui_wr_en <= '1';
            n_ui_rd_en <= '0';
            n_ui_adv   <= '1';
          when R_TWO =>
            n_ui_state <= ui_s_r_first;
            n_ui_wr_en <= '0';
            n_ui_rd_en <= '1';
            n_ui_adv   <= '0'; -- Disable commands while we process read 2nd cycle
          when R_ONE =>
            n_ui_state <= ui_s_r_one;
            n_ui_wr_en <= '0';
            n_ui_rd_en <= '1';
            n_ui_adv   <= '1';
          when IDLE =>
            n_ui_state <= ui_s_idle;
            n_ui_wr_en <= '0';
            n_ui_rd_en <= '0';
            n_ui_adv   <= '1';
          when others =>
            n_ui_state <= ui_s_idle;
            n_ui_wr_en <= '0';
            n_ui_rd_en <= '0';
            n_ui_adv   <= '0';
        end case;
    end case;
  end process;

  write_en <= true when (ui_wr_en = '1') else false;
  read_en  <= true when (ui_rd_en = '1') else false;

  mem_order_brc_g : if (MIG_COMMON.MEM_ADDR_ORDER = "BANK_ROW_COLUMN") generate
    -- MIG address app_addr is organised as RANK (unused), ROW, BANK, COL
    start_bank <= ui_addr(app_addr_width-2 downto AROW_WIDTH+ACOL_WIDTH);
    start_row  <= ui_addr(AROW_WIDTH+ACOL_WIDTH-1 downto ACOL_WIDTH);
    start_col  <= ui_addr(ACOL_WIDTH-1 downto 0);
  end generate;

  mem_order_rbc_g : if (MIG_COMMON.MEM_ADDR_ORDER = "ROW_BANK_COLUMN") generate
    -- MIG address app_addr is organised as RANK (unused), ROW, BANK, COL
    start_row  <= ui_addr(app_addr_width-2 downto ABAN_WIDTH+ACOL_WIDTH);
    start_bank <= ui_addr(ABAN_WIDTH+ACOL_WIDTH-1 downto ACOL_WIDTH);
    start_col  <= ui_addr(ACOL_WIDTH-1 downto 0);
  end generate;

  start_burst <= ui_addr(2 downto 0);

  -- 2 cycle accesses must always be BL8. 1st cycle offset = 0, 2nd cycle offset = 4
  start_cyc <= 4 when (ui_state = ui_s_w_second)or(ui_state = ui_s_r_second) else 0;

  -- 1 cycle accesses with DATA_WORDS = 4 must be BC4, otherwise BL8
  start_bl_cyc <= 2 when (DATA_WORDS = 4)and((ui_state = ui_s_w_one)or(ui_state = ui_s_r_one)) else 4;


  ------------------- Memory Log Section ----------------------

  log_memory : process(log_start)
    file     log_file  : text;
    variable file_line : line;
    variable temp_line : line;

    variable row_dec_width : natural;
    variable col_dec_width : natural;
    variable field_width   : natural;

    variable bank_i     : natural;
    variable row_i      : natural;
    variable col_i      : natural;

    variable byte_act   : natural;
    variable byte_undef : boolean;
    variable byte       : std_logic_vector(BYTE_WIDTH-1 downto 0);
  begin
    row_dec_width := conv_dec_width(AROW_WIDTH);
    col_dec_width := conv_dec_width(ACOL_WIDTH);

    -- Round up vector length to next nearest nibble
    if (BYTE_WIDTH mod 4 = 0) then
      field_width := BYTE_WIDTH/4;
    else
      field_width := BYTE_WIDTH/4 + 1;
    end if;
    field_width := 2 + field_width + 1;

    if (log_start) then
      open_file(log_filename,WRITE_MODE,log_file);
      assert false
        report "(mig_ddr3_lwb): Logging memory contents to " & log_filename & "..."
        severity note;

      byte_act := 0;

      for bank_i in 0 to (NUM_BANKS-1) loop
        report "Bank: " & integer'image(bank_i);
        for row_i in 0 to (NUM_ROWS-1) loop
          if (memory(bank_i)(row_i) /= NULL) then   -- Only process rows that are defined
            for col_i in 0 to (NUM_COLS-1) loop
              for byte_i in 0 to (DATA_BYTES-1) loop
                if (byte_act = 0) then              -- Start of new line
                  write(file_line,bank_i,right,1);            write(file_line,string'(" ")); -- Write start bank (decimal)
                  write(file_line,row_i,right,row_dec_width); write(file_line,string'(" ")); -- Write start row (decimal)
                  write(file_line,col_i,right,col_dec_width); write(file_line,string'(" ")); -- Write start col (decimal)
                  write(file_line,byte_i,right,1);            write(file_line,string'(" ")); -- Write start byte (decimal)
                end if;

                byte := memory(bank_i)(row_i)(col_i)(((byte_i+1)*BYTE_WIDTH)-1 downto byte_i*BYTE_WIDTH);
                hex_write(temp_line,byte,left,field_width,byte_undef);  -- Procedure defined in package

                if not(byte_undef) then             -- All bits valid 0 or 1
                  byte_act := byte_act + 1;
                  write(file_line,temp_line.all);
                else                                -- All bits undefined
                  if (byte_act > 0) then            -- Write line if it contains valid data
                    writeline(log_file,file_line);
                  else                              -- Delete line if it does not
                    deallocate(file_line);
                  end if;
                  byte_act := 0;
                end if;

                if (byte_act = BYTES_PER_LINE) then -- Write new line after 16 bytes output
                  writeline(log_file,file_line);
                  byte_act := 0;
                end if;
              end loop;
            end loop;
            -- Either write line or discard at end of active row
            if (byte_act > 0) then             -- Write line if it contains valid data
              writeline(log_file,file_line);
            else                               -- Delete line if it does not
              deallocate(file_line);
            end if;
            byte_act := 0;
          end if;
        end loop;
      end loop;

      file_close(log_file);
      assert false
        report "(mig_ddr3_lwb): Logging complete to " & log_filename
        severity note;
    end if;
  end process;

  ------------------- Memory Init Section ----------------------

  init_start_memory : process(init_start)
    file     init_file : text;
    variable file_line : line;

    variable read_ok        : boolean;
    variable bank_start_ok  : boolean;
    variable row_start_ok   : boolean;
    variable col_start_ok   : boolean;
    variable byte_start_ok  : boolean;
    variable end_of_line    : boolean;
    variable width_error    : boolean;

    variable col_cnt  : natural;
    variable byte_cnt : natural;

    variable bank_start_val : natural;
    variable row_start_val  : natural;
    variable col_start_val  : natural;
    variable byte_start_val : natural;

    variable byte : std_logic_vector(BYTE_WIDTH-1 downto 0);
  begin
    if (init_start) then
      open_file(init_filename,READ_MODE,init_file);
      assert false
        report "(mig_ddr3_lwb): Initialising memory contents from " & init_filename & "..."
        severity note;

      while not endfile(init_file) loop
        readline(init_file,file_line);

        read(file_line,bank_start_val,read_ok);
        if (read_ok)and(bank_start_val >= 0)and(bank_start_val < NUM_BANKS) then
          bank_start_ok := true;
        else
          bank_start_ok := false;
          assert false
            report "(mig_ddr3_lwb): ERROR: Illegal BANK start value " & integer'image(bank_start_val)
            severity error;
        end if;

        read(file_line,row_start_val,read_ok);
        if (read_ok)and(row_start_val >= 0)and(row_start_val < NUM_ROWS) then
          row_start_ok := true;
        else
          row_start_ok := false;
          assert false
            report "(mig_ddr3_lwb): ERROR: Illegal ROW start value " & integer'image(row_start_val)
            severity error;
        end if;

        read(file_line,col_start_val,read_ok);
        if (read_ok)and(col_start_val >= 0)and(col_start_val < NUM_COLS) then
          col_start_ok := true;
          col_cnt := col_start_val;
        else
          col_start_ok := false;
          assert false
            report "(mig_ddr3_lwb): ERROR: Illegal COL start value " & integer'image(col_start_val)
            severity error;
        end if;

        read(file_line,byte_start_val,read_ok);
        if (read_ok)and(byte_start_val >= 0)and(byte_start_val < DATA_BYTES) then
          byte_start_ok := true;
          byte_cnt := byte_start_val;
        else
          byte_start_ok := false;
          assert false
            report "(mig_ddr3_lwb): ERROR: Illegal BYTE start value " & integer'image(byte_start_val)
            severity error;
        end if;

        if (bank_start_ok and row_start_ok and col_start_ok and byte_start_ok) then
          byte_read(file_line,byte,end_of_line,width_error);
          while not(end_of_line) loop
            if (width_error) then
              assert false
                report "(mig_ddr3_lwb): ERROR: Illegal BYTE width at position (BANK|ROW|COL|BYTE) " &
                integer'image(bank_start_val) & " " &  integer'image(row_start_val) & " " &
                integer'image(col_cnt) & " " & integer'image(byte_cnt)
                severity error;
            else
              -- If row doesn't exist then create pointer to new row ready for writing
              if (memory(bank_start_val)(row_start_val) = NULL) then
                memory(bank_start_val)(row_start_val) := new col_array_t;
                init_row_cnt(bank_start_val) <= init_row_cnt(bank_start_val) + 1;
              end if;
              -- Initialise byte contents from file
              memory(bank_start_val)(row_start_val)(col_cnt)(((byte_cnt+1)*BYTE_WIDTH)-1 downto byte_cnt*BYTE_WIDTH) := byte;
              if (byte_cnt = (DATA_BYTES-1)) then
                byte_cnt := 0;
                col_cnt := col_cnt + 1;
              else
                byte_cnt := byte_cnt + 1;
              end if;
            end if;
            byte_read(file_line,byte,end_of_line,width_error);
          end loop;
        end if;
      end loop;

      file_close(init_file);
      assert false
        report "(mig_ddr3_lwb): Init complete from " & init_filename-- & " with " &
        severity note;
    end if;
  end process;

  ------------------- Memory Write Section ---------------------

  -- Perform write burst to memory
  -- Only perform write if we are in a valid FSM write cycle
  -- Only write to bytes enabled by data mask dm
  -- Write burst order determined by burst counter
  generate_write : process(clk)
      variable bank_i     : natural;
      variable row_i      : natural;
      variable burst_i    : natural;
      variable col_i      : natural;
      variable mask_in    : byte_s;
      variable data_in    : data_s;
      variable mask_i_top : natural;
      variable mask_i_bot : natural;
      variable data_i_top : natural;
      variable data_i_bot : natural;
      variable byte_i_top : natural;
      variable byte_i_bot : natural;
      variable data_hex   : string(1 to DATA_BYTES);
    begin
      if rising_edge(clk) then
        if write_en then
          memory_wr <= '1';
          for w in 0 to DATA_WORDS-1 loop
            bank_i  := to_integer(unsigned(start_bank));
            row_i   := to_integer(unsigned(start_row));
            burst_i := to_integer(unsigned(start_burst));
            col_i   := to_integer(unsigned(start_col)) + burst_wr_offset(burst_i,start_cyc + w,start_bl_cyc);

            -- Create pointer to new row ready for writing
            if (memory(bank_i)(row_i) = NULL) then
              memory(bank_i)(row_i) := new col_array_t;
              memory_row_cnt(w) <= memory_row_cnt(w) + 1;
            end if;

            data_i_top := DATA_WIDTH*(w+1)-1;
            data_i_bot := DATA_WIDTH*w;
            data_in := burst_data_d1(data_i_top downto data_i_bot);

            mask_i_top := DATA_BYTES*(w+1)-1;
            mask_i_bot := DATA_BYTES*w;
            mask_in := burst_mask_d1(mask_i_top downto mask_i_bot);

            for i in 0 to DATA_BYTES-1 loop
              -- Only write data byte if appropriate data mask bit is active
              byte_i_top := BYTE_WIDTH*(i+1)-1;
              byte_i_bot := BYTE_WIDTH*i;

              if (mask_in(i) = '0') then
                memory(bank_i)(row_i)(col_i)(byte_i_top downto byte_i_bot) := data_in(byte_i_top downto byte_i_bot);
              end if;
            end loop;

            memory_wr_mask(w) <= mask_in;
            memory_wr_data(w) <= data_in;
            memory_wr_bank(w) <= bank_i;
            memory_wr_row(w)  <= row_i;
            memory_wr_col(w)  <= col_i;

            for n in 0 to DATA_BYTES-1 loop
              data_hex(DATA_BYTES-n) := conv_nibble_hex(data_in(((n*4)+3) downto (n*4)));
            end loop;

            assert not(MESSAGE_WR)
              report "(mig_ddr3_lwb): DDR3 SDRAM Write : Bank = " & integer'image(bank_i) &
                     " Row = " & integer'image(row_i) &
                     " Col = " & integer'image(col_i) &
                     " Data(" & integer'image(data_i_top) & ":" & integer'image(data_i_bot) & ")" &
                     " = 0x" & data_hex
              severity note;
          end loop;
        else
          memory_wr <= '0';
          memory_wr_mask <= (others => (others => '-'));
          memory_wr_data <= (others => (others => '-'));
        end if;
      end if;
    end process;


  ------------------- Memory Read Section ----------------------

  -- Perform read burst from memory controlled by the burst counter
  generate_read : process(clk)
    variable bank_i      : natural;
    variable row_i       : natural;
    variable burst_i     : natural;
    variable col_i       : natural;
    variable data_out    : data_s;
    variable data_i_top  : natural;
    variable data_i_bot  : natural;
    variable data_hex    : string(1 to DATA_BYTES);
    variable row_pointer : row_pointer_t;
  begin

    if rising_edge(clk) then
      if read_en then
        memory_rd <= '1';
        for w in 0 to DATA_WORDS-1 loop
          bank_i  := to_integer(unsigned(start_bank));
          row_i   := to_integer(unsigned(start_row));
          burst_i := to_integer(unsigned(start_burst));
          col_i   := to_integer(unsigned(start_col)) + burst_rd_offset(burst_i,start_cyc + w,burst_type = "SEQ");

          row_pointer := memory(bank_i)(row_i);
          if (row_pointer = null) then
            data_out := (others => 'U');               -- Non-existent row - data_out undefined
          else
            data_out := memory(bank_i)(row_i)(col_i);  -- Existing row - access pointer
          end if;

          data_i_top := DATA_WIDTH*(w+1)-1;
          data_i_bot := DATA_WIDTH*w;
          data_out_full(data_i_top downto data_i_bot) <= data_out;

          memory_rd_data(w) <= data_out;
          memory_rd_bank(w) <= bank_i;
          memory_rd_row(w)  <= row_i;
          memory_rd_col(w)  <= col_i;

          for n in 0 to DATA_BYTES-1 loop
            data_hex(DATA_BYTES-n) := conv_nibble_hex(data_out(((n*4)+3) downto (n*4)));
          end loop;

          assert not(MESSAGE_RD)
            report "(mig_ddr3_lwb): DDR3 SDRAM Read  : Bank = " & integer'image(bank_i) &
                   " Row = " & integer'image(row_i) &
                   " Col = " & integer'image(col_i) &
                   " Data = 0x" & data_hex
            severity note;
        end loop;
      else
        memory_rd      <= '0';
        memory_rd_data <= (others => (others => '-'));
      end if;
    end if;
  end process;

end;


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

--
-- A synchronous FIFO used to buffer commands, command attributes, data, byte
-- enables etc. in 'mig_ddr3_lwb'.
--
-- 'ORDER' is log2 of the FIFO depth, e.g. ORDER = 4 => depth is 16 locations.
--
-- 'WIDTH' is the width of FIFO locations, in bits.
--
-- 'EMPTY_VAL' is the value driven on 'q(EMPTY_LO:EMPTY_HI-1)' when 'valid' is
-- not asserted; map both 'EMPTY_LO' and 'EMPTY_HI' to 0 if this feature is
-- not required.
--
entity mig_ddr3_lwb_cmd_fifo is
generic(
  ORDER       : in    natural range 2 to 6;
  WIDTH       : in    natural;
  EMPTY_VAL   : in    std_logic := '0';
  EMPTY_LO    : in    natural := 0;
  EMPTY_HI    : in    natural := 0);
port(
  rst         : in    std_logic;                             -- Asynchronous reset
  clk         : in    std_logic;                             -- Clock
  wr          : in    std_logic;                             -- '1' => write value on 'd' into FIFO
  d           : in    std_logic_vector(WIDTH - 1 downto 0);  -- Item to write into FIFO, qualified by 'wr'
  adv         : in    std_logic;                             -- '1' => present next item on 'q' to next item
  full        : out   std_logic;                             -- '1' => full
  not_full    : out   std_logic;                             -- '1' => not full
  valid       : out   std_logic;                             -- '1' => not empty
  q           : out   std_logic_vector(WIDTH - 1 downto 0)); -- Current item, qualified by 'valid'
end;

architecture synth of mig_ddr3_lwb_cmd_fifo is

  constant DEPTH : natural := 2**ORDER;
  constant MAX_LEVEL : natural := DEPTH - 1;

  -- Generate initial value for rq output
  constant RQ_TOP  : std_logic_vector(WIDTH - 1 downto EMPTY_HI)    := (others => '-');
  constant RQ_MID  : std_logic_vector(EMPTY_HI - 1 downto EMPTY_LO) := (others => EMPTY_VAL);
  constant RQ_LOW  : std_logic_vector(EMPTY_LO - 1 downto 0)        := (others => '-');
  constant RQ_INIT : std_logic_vector(WIDTH - 1 downto 0) := RQ_TOP & RQ_MID & RQ_LOW;

  subtype ram_data_t is std_logic_vector(WIDTH - 1 downto 0);
  type ram_array_t is array(0 to DEPTH - 1) of ram_data_t;

  signal wptr : std_logic_vector(ORDER - 1 downto 0) := (others => '0');
  signal rptr : std_logic_vector(ORDER - 1 downto 0) := (others => '0');
  signal rn : std_logic_vector(ORDER - 1 downto 0) := (others => '0');
  signal not_empty : std_logic;
  signal i_full, i_not_full : std_logic;

  signal qvalid : std_logic := '0';
  signal qce : std_logic;
  signal rd : std_logic;
  signal i_q : std_logic_vector(WIDTH - 1 downto 0) := RQ_INIT;

  signal ram : ram_array_t;
  attribute RAM_STYLE : string;
  attribute RAM_STYLE of ram : signal is "distributed";
  signal sel_ram : ram_data_t;

begin

  valid <= qvalid;
  full <= i_full;
  not_full <= i_not_full;
  q <= i_q;

  write_logic : process(rst, clk)
    variable tmp : std_logic_vector(ORDER - 1 downto 0);
  begin
    if rst = '1' then
      wptr <= (others => '0');
      i_not_full <= '1';
      i_full <= '0';
    elsif rising_edge(clk) then
      if wr = '1' then
        wptr <= wptr + 1;
      end if;

      if wr = '1' then
        assert CONV_INTEGER(wptr - rptr) < MAX_LEVEL severity failure;
        tmp := wptr - rptr + 1;
        i_full <= tmp(ORDER - 1);
        i_not_full <= NOT tmp(ORDER - 1);
      else
        tmp := wptr - rptr;
        i_full <= tmp(ORDER - 1);
        i_not_full <= NOT tmp(ORDER - 1);
      end if;
    end if;

    if rising_edge(clk) then
      if wr = '1' then
        ram(conv_integer(wptr)) <= d;
      end if;
    end if;
  end process;

  not_empty <= OR_reduce(rn);
  qce <= adv OR NOT qvalid;
  rd <= (adv OR NOT qvalid) AND not_empty;

  read_logic : process(rst, clk)
  begin
    if rst = '1' then
      rptr <= (others => '0');
    elsif rising_edge(clk) then
      if rd = '1' then
        rptr <= rptr + 1;
      end if;
    end if;

    if rst = '1' then
      null;
    elsif rising_edge(clk) then
      if rd = '1' then
        assert CONV_INTEGER(wptr - rptr) > 0 severity failure;
      end if;
    end if;

    if rising_edge(clk) then
      if rd = '1' then
        rn <= wptr - rptr - 1;
      else
        rn <= wptr - rptr;
      end if;
    end if;
  end process;

  output_valid : process(rst, clk)
  begin
    if rst = '1' then
      qvalid <= '0';
    elsif rising_edge(clk) then
      if qce = '1' then
        qvalid <= not_empty;
      end if;
    end if;
  end process;

  sel_ram <= ram(conv_integer(rptr));

  output_data : for i in 0 to WIDTH - 1 generate
    with_reset : if EMPTY_LO <= i AND i < EMPTY_HI generate
      U0 : process(rst, clk)
      begin
        if rst = '1' then
          i_q(i) <= EMPTY_VAL;
        elsif rising_edge(clk) then
          if qce = '1' then
            if not_empty = '1' then
              i_q(i) <= sel_ram(i);
            else
              i_q(i) <= EMPTY_VAL;
            end if;
          end if;
        end if;
      end process;
    end generate;

    without_reset : if NOT(EMPTY_LO <= i AND i < EMPTY_HI) generate
      U0 : process(clk)
      begin
        if rising_edge(clk) then
          if qce = '1' then
            -- Assign i_q to undefined on FIFO empty to assist in simulation debug only
            if not_empty = '1' then
              i_q(i) <= sel_ram(i);
            else
              i_q(i) <= '-';
            end if;
          end if;
        end if;
      end process;
    end generate;
  end generate;

end;