#
# makeppr.tcl - Creates a PlanAhead project for an example FPGA design.
#
# This script is intended to be sourced by another TCL script running in
# PlanAhead's TCL interpreter.
#
# Expects the following variables to be defined:
#
# design          e.g. "simple"
# model           e.g. "admxrc6tl"
# top             e.g. "${design}"
# sim_top         e.g. "test_${design}"
# device          e.g. "6vlx240t"
# package         e.g. "ff1759"
# speed           e.g. "1"
# step            e.g. "ies"
# syn_src         list of synthesis HDL sources
# constraint      list of constraint sources
# src_sim_ocp     list of OCP simulation sources, or empty list
# src_sim_lwb     list of OCP LWB simulation sources, or empty list
# src_sim_mptl    list of MPTL simulation sources, or empty list
#
# Expects the XILINX_PLANAHEAD environment variable to be defined as per
# settings32.bat, settings64.bat, settings32.sh or settings64.sh.
#
# Changelog
# ---------
#
# 30 Jul 2012  TW   Added support to detect PlanAhead version so that it works
#                   with both PlanAhead 13.4 & 14.2.
# 05 Nov 2012 KDC   Added support for device stepping e.g. ies, ges using step variable
#
# 08 Nov 2012  TW   Now also works with PlanAhead 14.3.
#

set planahead $env(XILINX_PLANAHEAD)
set project_dir "./planahead"

set planahead_version [lindex [version] 1]
if { [ regexp -nocase v\[0-9\]+\.\[0-9\]+ ${planahead_version} ] } {
  set planahead_major [ lindex [split ${planahead_version} "."] 0 ]
  set planahead_major [ regsub {v([0-9]+)} ${planahead_major} {\1} ]
  set planahead_minor [ lindex [split ${planahead_version} "."] 1 ]
} else {
  error "PlanAhead version is not in expected form 'v<major>.<minor>'"
}
puts "PlanAhead version is ${planahead_major}.${planahead_minor}"

set part "xc${device}${package}-${speed}"

if { [ info exists step ] } {
  puts "Creating project ${project_dir}/${design}-${model}-${device}_${step}.ppr"
  set project [ create_project -part $part -force "${design}-${model}-${device}_${step}" ${project_dir} ]
} else {
  puts "Creating project ${project_dir}/${design}-${model}-${device}.ppr"
  set project [ create_project -part $part -force "${design}-${model}-${device}" ${project_dir} ]
}

# Set this because PlanAhead doesn't always succeed in figuring out a correct
# compile order. We want it to use the order that we add source files as the
# compile order.
set_property source_mgmt_mode None ${project}
set_property target_language VHDL ${project}

set fileset_syn [ get_fileset sources_1 ]
if { [ llength ${src_sim_ocp}  ] } { set fileset_sim_ocp  [ create_fileset -simset sim_ocp ] }
if { [ llength ${src_sim_lwb}  ] } { set fileset_sim_lwb  [ create_fileset -simset sim_lwb ] }
if { [ llength ${src_sim_mptl} ] } { set fileset_sim_mptl [ create_fileset -simset sim_mptl ] }

foreach src ${src_syn} {
  add_files -fileset sources_1 -norecurse ${src}
}
set_property used_in_simulation false [get_files -of_objects sources_1]
set_property top ${top} ${fileset_syn}

if { [ llength ${src_sim_ocp} ] } {
  foreach src ${src_sim_ocp} {
    add_files -fileset sim_ocp -norecurse ${src}
  }
  set_property used_in_synthesis false [get_files -of_objects sim_ocp]
  set_property used_in_simulation true [get_files -of_objects sim_ocp]
  if { ${planahead_major} < 14 || (${planahead_major} == 14 && ${planahead_minor} < 3) } {
    set_property used_in_implementation false [get_files -of_objects sim_ocp]
  }
  set_property top ${sim_top} ${fileset_sim_ocp}
  set_property runtime {10 ms} ${fileset_sim_ocp}
  set_property ng.output_hdl_format VHDL ${fileset_sim_ocp}
  set_property FUSE.LOAD_GLBL 1 ${fileset_sim_ocp}
  if { [ info exists vlog_defs_sim_ocp ] } {
    set_property VERILOG_DEFINE ${vlog_defs_sim_ocp} ${fileset_sim_ocp}
  }
  if { [ info exists top_sim_ocp ] } {
    set_property top ${top_sim_ocp} ${fileset_sim_ocp}
  }
}

if { [ llength ${src_sim_lwb} ] } {
  foreach src ${src_sim_lwb} {
    add_files -fileset sim_lwb -norecurse ${src}
  }
  set_property used_in_synthesis false [get_files -of_objects sim_lwb]
  set_property used_in_simulation true [get_files -of_objects sim_lwb]
  if { ${planahead_major} < 14 || (${planahead_major} == 14 && ${planahead_minor} < 3) } {
    set_property used_in_implementation false [get_files -of_objects sim_lwb]
  }
  set_property top ${sim_top} ${fileset_sim_lwb}
  set_property runtime {10 ms} ${fileset_sim_lwb}
  set_property ng.output_hdl_format VHDL ${fileset_sim_lwb}
  set_property FUSE.LOAD_GLBL 1 ${fileset_sim_lwb}
  if { [ info exists vlog_defs_sim_lwb ] } {
    set_property VERILOG_DEFINE ${vlog_defs_sim_lwb} ${fileset_sim_lwb}
  }
  if { [ info exists top_sim_lwb ] } {
    set_property top ${top_sim_lwb} ${fileset_sim_lwb}
  }
}

if { [ llength ${src_sim_mptl} ] } {
  foreach src ${src_sim_mptl} {
    add_files -fileset sim_mptl -norecurse ${src}
  }
  set_property used_in_synthesis false [get_files -of_objects sim_mptl]
  set_property used_in_simulation true [get_files -of_objects sim_mptl]
  if { ${planahead_major} < 14 || (${planahead_major} == 14 && ${planahead_minor} < 3) } {
    set_property used_in_implementation false [get_files -of_objects sim_mptl]
  }
  set_property top ${sim_top} ${fileset_sim_mptl}
  set_property runtime {10 ms} ${fileset_sim_mptl}
  set_property ng.output_hdl_format VHDL ${fileset_sim_mptl}
  set_property FUSE.LOAD_GLBL 1 ${fileset_sim_mptl}
  if { [ info exists vlog_defs_sim_mptl ] } {
    set_property VERILOG_DEFINE ${vlog_defs_sim_mptl} ${fileset_sim_mptl}
  }
  if { [ info exists top_sim_mptl ] } {
    set_property top ${top_sim_mptl} ${fileset_sim_mptl}
  }
}

foreach src ${constraint} {
  add_files -fileset constrs_1 -norecurse ${src}
}
set xst_opts "-read_cores optimize"

foreach dir ${core_dir} {
  set xst_opts "${xst_opts} -sd ${dir}"
}
set xst_opts "${xst_opts} -write_timing_constraints yes -reduce_control_sets auto"

config_run synth_1 -program xst -option {More Options} -value "${xst_opts}"
config_run synth_1 -program xst -option -opt_mode -value {speed}
config_run synth_1 -program xst -option -opt_level -value {2}
config_run synth_1 -program xst -option -lc -value {auto}
config_run impl_1 -program map -option -ol -value {high}
config_run impl_1 -program map -option -pr -value {b}
config_run impl_1 -program map -option -timing -value {true}
config_run impl_1 -program par -option -ol -value {high}
if { ${planahead_major} >= 14 } {
  set_property steps.trce.args.e {3} [get_runs impl_1]
  set_property steps.trce.args.l {0} [get_runs impl_1]
  set_property steps.trce.args.u {3} [get_runs impl_1]
  set_property steps.trce.args.v {3} [get_runs impl_1]
  # It is possible to set sticky bitgen options in ISE 14 and later
  set bitgen_common_opt {-g drivedone:yes -g unusedpin:pullnone -g compress}
  if { [ info exists bitgen_extra_opt ] } {
    set bitgen_opt "${bitgen_common_opt} ${bitgen_extra_opt}"
  } else {
    set bitgen_opt "${bitgen_common_opt}"
  }
  set_property -name {steps.bitgen.args.More Options} -value "${bitgen_opt}" -objects [get_runs impl_1]
} else {
  config_run impl_1 -program trce -option -e -value {3}
  config_run impl_1 -program trce -option -l -value {0}
  config_run impl_1 -program trce -option -u -value {3}
  config_run impl_1 -program trce -option -v -value {3}
}

close_project
