--==============================================================================
--
--     File Name: blk_mem_if_admxrc7k1.vhd
--       Version: 0.2
--          Date: 13 Nov 2012
--         Model: Synthesisable
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: On-board RAM interface block
--
--   Description: OCP to on-board RAM interface
--
--   Limitations: Uses Coregen MIG DDR3 SDRAM bank block ddr3_if_bank
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2012 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 19 Jan 2012: Initial design
--                0.2, KDC, 13 Nov 2012:
--                     - Added clock generic to ddr3_if_bank instances
--                     - Added constant MIG_CLOCKS
--                     - Added function conv_clock_option
--                     - Added new ports ddr3_xadc_temp_in, ddr3_xadc_temp_out
--                       which are needed for MIG7 1.8 onwards
--
--==============================================================================


library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;
use work.ddr3_if_pkg.all;
use work.uber_pkg.all;

entity blk_mem_if is
port(
  -- OCP interface
  ocp_rst         : in    std_logic;
  ocp_clk         : in    std_logic;
  mem_if_m2s      : in    adb3_ocp_m2sT_array(0 to MEM_BANKS-1);
  mem_if_s2m      : out   adb3_ocp_s2mT_array(0 to MEM_BANKS-1);
  -- Memory clocking
  mem_rst         : in    std_logic;
  mem_ref_clk     : in    std_logic;
  mem_clk         : in    mem_clk_array_t;
  -- Memory status
  mem_if_rdy      : out   mem_if_rdy_array_t;
  mem_if_stat     : out   mem_if_stat_array_t;
  mem_if_err      : out   mem_if_err_array_t;
  -- Memory physical interface
  mem_addr_out    : out   mem_addr_out_t;
  mem_ctrl_out    : out   mem_ctrl_out_t;
  mem_data_inout  : inout mem_data_inout_t;
  mem_clk_out     : out   mem_clk_out_t;
  -- Debug info
  mem_if_err_info : out   mem_if_debug_array_t);
end;


architecture syn_admxrc7k1 of blk_mem_if is

  -- Function to select DDR3 SDRAM speed bin
  function conv_clock_option(
    constant val: natural)
  return mig_clocks_t is
  begin
    assert (val = 14)or(val = 15)
      report "(conv_clock_option): DDR3 row width " & natural'image(val) & " is out of range 14..15."
      severity failure;

    case val is
      when 14     => return MIG_CLOCKS_1600; -- DDR3 1600 for 2 Gib devices
      when 15     => return MIG_CLOCKS_800;  -- DDR3 800  for 4 Gib devices
      when others => return MIG_CLOCKS_1600;
    end case;
  end;

  -- DDR3_BANK_ROW_WIDTH constant package adb3_target_inc_pkg
  -- conv_*_option function package ddr3_if_pkg
  constant MIG_CLOCKS : mig_clocks_t := conv_clock_option(DDR3_BANK_ROW_WIDTH);

  -- DDR3 bank reset
  -- Comment from coregen MIG DDR3 SDRAM interface iodelay_ctrl block
  -- Clock cycles to delay deassertion of reset. Needs to be a fairly
  -- high number not so much for metastability protection, but to give time
  -- for reset (i.e. stable clock cycles) to propagate through all state
  -- machines and to all control signals (i.e. not all control signals have
  -- resets, instead they rely on base state logic being reset, and the effect
  -- of that reset propagating through the logic). Need this because we may not
  -- be getting stable clock cycles while reset asserted (i.e. since reset
  -- depends on DCM lock status)
  -- COMMENTED, RC, 01/13/09 - causes pack error in MAP w/ larger #
  constant RST_SYNC_NUM : integer := 15;

  signal rst_ref        : std_logic;
  signal rst_ref_sync_r : std_logic_vector(RST_SYNC_NUM-1 downto 0);

  -- DDR3 bank infrastructure signals
  signal iodelay_ctrl_rdy : std_logic;

  -- DDR3 attributes
  attribute IODELAY_GROUP : string;
  attribute IODELAY_GROUP of idelayctrl_i : label is MIG_CLOCKS_COMMON.IODELAY_GRP;

  attribute syn_maxfan : integer;
  attribute syn_maxfan of rst_ref_sync_r : signal is 10;

  -- DDR3 XADC temperature signals
  type mem_xadc_temp_t is array(0 to DDR3_BANKS-1) of std_logic_vector(11 downto 0);
  signal mem_xadc_temp_in  : std_logic_vector(11 downto 0);
  signal mem_xadc_temp_out : mem_xadc_temp_t;

begin

  --*****************************************************************
  -- IDELAYCTRL reset
  -- This assumes an external clock signal driving the IDELAYCTRL
  -- blocks. Otherwise, if a PLL drives IDELAYCTRL, then the PLL
  -- lock signal will need to be incorporated in this.
  --*****************************************************************

  process(mem_rst,mem_ref_clk)
  begin
    if (mem_rst = '1') then
      rst_ref_sync_r <= (others => '1');
    elsif rising_edge(mem_ref_clk) then
      rst_ref_sync_r <= std_logic_vector(unsigned(rst_ref_sync_r) sll 1);
    end if;
  end process;

  rst_ref <= rst_ref_sync_r(RST_SYNC_NUM-1);

  idelayctrl_i : IDELAYCTRL
  port map (
    RDY    => iodelay_ctrl_rdy,  -- out
    REFCLK => mem_ref_clk,       -- in
    RST    => rst_ref);          -- in


  -- Connect XADC temperature output from bank 0 to all banks
  mem_xadc_temp_in <= mem_xadc_temp_out(0);

  -- Instantiate DDR3 SDRAM memory banks
  -- Generic sim is true for simulation and omitted (false) for synthesis
  -- Assign mapping of OCP channel number to memory bank
  -- DDR3_BANKS, DDR3_BANK0 constants package adb3_target_inc_pkg
  ddr3_if_g : for n in 0 to DDR3_BANKS-1 generate
    ddr3_if_bank_i : ddr3_if_bank
    generic map(
      -- synthesis translate_off
      sim   => true,       -- boolean := false
      -- synthesis translate_on
      clock => MIG_CLOCKS, -- mig_clocks_t := MIG_CLOCKS_1600
      bank  => n)          -- integer range 0 to DDR3_BANKS-1 := 0)
    port map(
      -- OCP Interface
      ocp_rst               => ocp_rst,                           -- in    std_logic
      ocp_clk               => ocp_clk,                           -- in    std_logic
      ocp_m2s               => mem_if_m2s(DDR3_BANK0 + n),        -- in    adb3_ocp_m2sT
      ocp_s2m               => mem_if_s2m(DDR3_BANK0 + n),        -- out   adb3_ocp_s2mT
      -- DDR3 Clocking
      ddr3_rst              => mem_rst,                           -- in    std_logic
      ddr3_ref_clk          => mem_ref_clk,                       -- in    std_logic
      ddr3_clk              => mem_clk(DDR3_BANK0 + n),           -- in    std_logic
      ddr3_iodelay_ctrl_rdy => iodelay_ctrl_rdy,                  -- in    std_logic
      ddr3_xadc_temp_in     => mem_xadc_temp_in,                  -- in    std_logic_vector(11 downto 0);
      ddr3_xadc_temp_out    => mem_xadc_temp_out(n),              -- out   std_logic_vector(11 downto 0);
      -- DDR3 Status
      ddr3_if_rdy           => mem_if_rdy(DDR3_BANK0 + n),        -- out   std_logic
      ddr3_if_stat          => mem_if_stat(DDR3_BANK0 + n),       -- out   std_logic_vector(3 downto 0)
      ddr3_if_err           => mem_if_err(DDR3_BANK0 + n),        -- out   std_logic_vector(3 downto 0)
      -- DDR3 Physical Interface
      ddr3_addr_out         => mem_addr_out.ddr3_addr_out(n),     -- out   ddr3_addr_out_t
      ddr3_ctrl_out         => mem_ctrl_out.ddr3_ctrl_out(n),     -- out   ddr3_ctrl_out_t
      ddr3_data_inout       => mem_data_inout.ddr3_data_inout(n), -- inout ddr3_data_inout_t
      ddr3_clk_out          => mem_clk_out.ddr3_clk_out(n),       -- out   ddr3_clk_out_t
      -- DDR3 Interface Error info
      ddr3_if_debug         => mem_if_err_info(DDR3_BANK0 + n));  -- out   std_logic_vector(31 downto 0))
  end generate;

end;